(***************************************************************************
  FreeHC program
  A free help compiler for Borland's TPH format
  Copyright Duncan Murdoch and Peter Brandstrom

  Written by Peter Brandstrom, January 1994

  Command line parsing and TPH output by Duncan Murdoch.
  Help text source parsing by Peter Brandstrom.

***************************************************************************)
program FreeHC;

  uses
    Dos, Objects,
    OPString,
    HelpFiles, THelp, TokenUnit,
    HC, TextStrm;

  const
    { Set to true for TASM look-alike errors, bug in BP 7.0 unfortunately }
    TasmErrors   = False;       

  type
    PFreeHelpCompiler = ^TFreeHelpCompiler;
    TFreeHelpCompiler =
      object (THelpCompiler)
        { A help compiler for Dos command line use }
        procedure Message(const MsgType, FileName:String; aLine:Word; const Msg:String); virtual;
          { Print an error message }
        procedure Progress; virtual;
          { Print progress information }
      end;


  var
    HelpFileName : String;
    HelpTextName : String;
    IncludeDirs  : String;
    Quiet        : Boolean;


(***************************************************************************
***************************************************************************)

  function  DefaultExtension(const FileName, Ext:String; Force:Boolean):String;
    var
      P: PathStr;
      D: DirStr;
      N: NameStr;
      E: ExtStr;
  begin
    FSplit(FileName, D, N, E);
    if (E = '') or Force then
      E:=Ext;
    DefaultExtension:=D+N+E;
  end;

  function JustTheName(const FileName:String):String;
    var
      P: PathStr;
      D: DirStr;
      N: NameStr;
      E: ExtStr;
  begin
    FSplit(FileName, D, N, E);
    JustTheName:=N;
  end;


(***************************************************************************
***************************************************************************)

  procedure TFreeHelpCompiler.Message;
    { Output a message }
  begin
    if TasmErrors then                                     { TASM style }
      WriteLn('**Error** ', FileName, '(', aLine, ') ', Msg)
    else                                                   { TVHC style }
      WriteLn(MsgType, FileName, '(', aLine, '): ', Msg);
  end;


  procedure TFreeHelpCompiler.Progress;
    { Progress indication. Called when Pass 1 and Pass 2 begins, and
      after every topic compiled }
    const
      LastTime : Word = 0;
  begin
    if not Quiet then
    begin
      if not Pass2 then
        WriteLn('Pass 1...')
      else
        if TopicsCompiled=0 then
          WriteLn('Pass 2...');

     {$IFOPT Q+} {$DEFINE QOpt} {$ENDIF} {$Q-}
      if MemW[Seg0040:$6C]-LastTime > 5 then
     {$IFDEF QOpt} {$Q+} {$UNDEF QOpt} {$ENDIF}
      begin
        if TopicsToCompile>0 then
          Write(LongDiv(LongMul(50, TopicsCompiled), TopicsToCompile):2,'% complete'#13);
        LastTime:=MemW[Seg0040:$6C];
      end;
    end;
  end;

(***************************************************************************
***************************************************************************)

  procedure WriteCopyright;
    { Display copyright notice }
  begin
    WriteLn('Free Help Compiler 1.0 Copyright (c) D.J. Murdoch, P. Brandstrom 1994');
{    WriteLn('Portions copyright TurboPower Software');  Not in my version! }
  end;


  procedure WriteHelp;
    { Summarize usage }
  begin
    WriteLn;
    WriteLn('Usage: FREEHC [options] file');
    WriteLn;
    WriteLn('FreeHC compiles help files into Borland''s TPH format');
    WriteLn;
    WriteLn('Options:');
{    Writeln('  /FB           write Borland .TPH format');
    Writeln('  /FH           write HELPC .HDF format');
    WriteLn('  /FP           write MAKEHELP .TXT format');
    Writeln('  /FT           write TeX .TEX format');     }
    WriteLn('  /I dirs       Search list for include files');
    WriteLn('  /O filename   Output filename');
    WriteLn('  /Q            Quiet, show errors only');
    Halt;
  end;


  procedure UnknownCommand(const Arg:String);
    { Complain about bad command line }
  begin
    WriteLn;
    WriteLn('Unknown command line argument ', Arg);
    WriteHelp;
  end;


  procedure ParseCommandLine;
    { Get command line switches }
  var
    Arg    : String[127];
    ArgNum : Word;
    i      : Integer;

    function ArgSuffix(const Msg:String):String;
      { Returns the suffix to an argument that takes one }
    begin
      if Length(Arg) > 2 then
        ArgSuffix := Copy(Arg,3,255)
      else if ArgNum >= ParamCount then
        WriteLn(Msg+' missing')
      else begin
        Inc(ArgNum);
        ArgSuffix := Paramstr(ArgNum);
      end;
    end;

  begin
    { If no parms, just display help }
    If ParamCount = 0 then
      WriteHelp;

    HelpFileName := '';
    ArgNum := 1;
    while ArgNum <= ParamCount do begin
      Arg := ParamStr(ArgNum);
      case Arg[1] of
        '/', '-' : {Option identifier}
          case Upcase(Arg[2]) of
{            'F' : if length(Arg) = 3 then
                    case Upcase(Arg[3]) of
                      'B' : begin
                              HelpFormat := BorHelpFormat;
                              FileExtension := 'TPH';
                            end;
                      'H' : begin
                              HelpFormat := HDKHelpFormat;
                              FileExtension := 'HDF';
                            end;
                      'P' : begin
                              HelpFormat := PopHelpFormat;
                              FileExtension := 'TXT';
                            end;
                      'T' : begin
                              HelpFormat := TexHelpFormat;
                              FileExtension := 'TEX';
                            end;
                      else UnknownCommand(Arg);
                    end
                    else
                      UnknownCommand(Arg);     }
            'I' : IncludeDirs:=IncludeDirs+';'+Copy(ParamStr(ArgNum), 3, 255);
            'O' : HelpFileName:=ArgSuffix('Output filename');
            'Q' : Quiet:=True;
          else
            UnknownCommand(Arg);
          end
        else
          HelpTextName := stUpcase(DefaultExtension(Arg, '.TXT', False));
      end;
      Inc(ArgNum);
    end;
  end;


(***************************************************************************
***************************************************************************)

  var
    H : TFreeHelpCompiler;
    HF : PBorHelpFile;
    S : PBufStream;
    HelpText : PTextStream;
    Size, UncompressedSize : Longint;


  procedure MyNotifyProc(Num:Longint); far;
    const
      LastTime : Word = 0;
  begin
   {$IFOPT Q+} {$DEFINE QOpt} {$ENDIF} {$Q-}
    if MemW[Seg0040:$6C]-LastTime > 5 then
   {$IFDEF QOpt} {$Q+} {$UNDEF QOpt} {$ENDIF}
    begin
      if H.TopicsToCompile>0 then
        Write(50+LongDiv(LongMul(50, Num), H.TopicsToCompile),'% complete'#13);
      LastTime:=MemW[Seg0040:$6C];
    end;
  end;


begin
  WriteCopyright;
  ParseCommandLine;

  if HelpTextName='' then
    WriteHelp;

  if HelpFileName='' then
    HelpFileName:=JustTheName(HelpTextName);

  HelpFileName:=DefaultExtension(HelpFileName, '.TPH', False);

  New(HF, InitNew(1, NoCompression, JustTheName(HelpTextName)));

  New(HelpText, Init(HelpTextName, stOpenRead, 8000));
  H.Init(HelpTextName, HelpText, HF, IncludeDirs);

  if HelpText^.Status<>stOK then
    H.Error('Can''t open source file')
  else
    if HF=Nil then
      H.Error('Can''t create output file')
    else
    begin
      H.Compile;

      UncompressedSize:=400+HF^.Base^.GetSize+HF^.Contexts^.Num*3;
      New(S, Init(HelpFileName, stCreate, 2048));
      if S^.Status<>stOK then
        H.Error('Can''t create output file')
      else
      begin
        if not Quiet then
        begin
          WriteLn('Compressing...');
          NotifyProc:=MyNotifyProc;
        end;
        HF^.Rewrite(S);

        if not Quiet then
        begin
          Size:=S^.GetSize;
          WriteLn(Size, ' bytes written, ',
            (UncompressedSize-Size)*100 div Size, '% compression');
        end;
      end;
    end;

  H.Done;
  Tokens.Done;
end.