(***************************************************************************
  TextStream unit
  Buffered stream that reads lines of text fast
  PJB September 22, 1993, Internet mail to d91-pbr@nada.kth.se
  Copyright 1993, All Rights Reserved
  Free source, use at your own risk.
  If modified, please state so if you pass this around.

  TextStream is rather fast and rather dumb, seeks more than necessary.
  A line ends with a Line feed. The preceding character is assumed to
  be a Carriage return.

  LineLength never returns a value greater than the current buffer size
  which means that lines longer than the buffer size will be split into
  several lines.

  ReadLine will split lines longer than 255 characters.

***************************************************************************)
unit TextStrm;

{$R-,S-}

interface

  uses
    Objects;

  type
    PTextStream = ^TTextStream;
    TTextStream =
      object (TBufStream)
        procedure GetText(var Buf; Count:word);
        function  LineLength:word;
        procedure ReadLine(var S:String);
        procedure WriteLine(const S:String);
      end;


(***************************************************************************
***************************************************************************)
implementation


  (*******************************************************************
    Get Count number of characters from stream, skip CrLf
  *******************************************************************)
  procedure TTextStream.GetText; assembler;
  asm
      push ds

      mov  cx,Count

      lds  si,Self
      mov  ax,[si].BufPtr

      mov  bx,ax
      add  bx,cx
      add  bx,2
      cmp  bx,[si].BufEnd
      jbe  @OK
      mov  bx,[si].BufEnd
    @OK:
      mov  [si].BufPtr,bx

      lds  si,[si].Buffer
      add  si,ax

      cld

      les  di,Buf

      shr  cx,1
      rep  movsw
      rcl  cx,1
      rep  movsb

      pop  ds
  end;


  (*******************************************************************
    Calculate length of next line in buffer
  *******************************************************************)
  function TTextStream.LineLength; assembler;
  asm
    @Start:
      les  di,Self
      mov  ax,es:[di].BufPtr
      cmp  ax,es:[di].BufEnd
      jb   @NotEmpty

      call @More
      jmp  @NotEmpty

    @More:
      push WORD PTR es:[di].Buffer[2]
      push WORD PTR es:[di].Buffer[0]
      mov  ax,1
      push ax
      push es
      push di

      call Read

      les  di,Self
      cmp  es:[di].Status,0
      jne  @Error
      dec  es:[di].BufPtr
      retn

    @Error:
      pop  bx
      xor  ax,ax
      jmp  @End

    @NotEmpty:
      mov  ax,es:[di].BufPtr
      mov  cx,es:[di].BufEnd
      sub  cx,ax
      mov  dx,cx

      les  di,es:[di].Buffer
      add  di,ax

      mov  al,10
      cld
      repne scasb
      je   @Fin


      push dx

      les  di,Self
      push es
      push di
      call GetPos

      push dx
      push ax
      les  di,Self
      mov  es:[di].BufPtr,0
      mov  es:[di].BufEnd,0
      push es
      push di
      call Seek

      call @More

      pop  ax
      cmp  ax,es:[di].BufEnd
      jae  @End
      jmp  @NotEmpty

    @Fin:
      mov  ax,dx
      sub  ax,cx
      sub  ax,2

    @End:
  end;


  (*******************************************************************
    Read line from stream into string
  *******************************************************************)
  procedure TTextStream.ReadLine;
    var
      Count : word;
      OverFlow : boolean;
  begin
    Count:=LineLength;
    OverFlow:=Count>255;
    if OverFlow then
      Count:=255;

    S[0]:=Chr(Count);
    GetText(S[1], Count);

    if OverFlow then
      Dec(BufPtr, 2);
  end;


  (*******************************************************************
    Write one line to stream
  *******************************************************************)
  procedure TTextStream.WriteLine;
    const
      CrLf : array [1..2] of char = #13#10;
  begin
    Write(PString(@S[1])^, Length(S));   (* Bug in OBJECTS.PAS *)
    Write(CrLf, 2);
  end;


    (*******************************************************************
    *******************************************************************)

end.